@@ -2,7 +2,7 @@ package Padre;
 use strict;
 use warnings;
 
-our $VERSION = '0.03';
+our $VERSION = '0.04';
 
 =head1 NAME
 
@@ -10,6 +10,9 @@ Padre - Perl Application Development and Refactoring Environment
 
 =head1 SYNOPSIS
 
+Padre is a text editor aimed to be an IDE for Perl.
+
+
 You should be able to just type in 
 
  padre
@@ -85,17 +88,37 @@ You can edit the command line using the Run/Setup menu item.
  padre --index
 
 
-
 =head1 Command line options
 
  --index   will go over the @INC and list all the available modules in the database
  
  a list of filenames can be given to be opened
 
+=head1 Plugins
+
+There is a highly experimental but quit simple plugin system.
+
+A plugin is a module in the Padre::Plugin::* namespace.
+
+At startup time Padre looks for all such modules in @INC 
+and loads them.
+Every plugin must have a C<menu> method that returns its menu items
+which is a list of lists:
+
+ ( 
+   [ Name_1, \&callback_1 ],
+   [ Name_2, \&callback_2 ],
+ )
+
+Padre will add a menu entry for every plugin under the B<Plugins>
+menu item. For each plugin menu item it will add all the Name_1,
+Name_2 subitems.
+
+
 =cut
 
 use File::HomeDir         qw();
-use File::Spec::Functions qw(catfile);
+use File::Spec::Functions qw(catfile catdir);
 use DBI                   qw();
 use Carp                  qw();
 use YAML                  qw(LoadFile DumpFile);
@@ -118,6 +141,7 @@ sub new {
     $self->{recent}{$_} = [] for @history;
 
     $self->_process_command_line;
+    $self->_locate_plugins;
 
     return $self;
 }
@@ -161,6 +185,31 @@ sub _process_command_line {
     return;
 }
 
+sub _locate_plugins {
+    my ($self) = @_;
+    my %plugins;
+    foreach my $path (@INC) {
+        my $dir = catdir($path, 'Padre', 'Plugin');
+        opendir my $dh, $dir or next;
+        while (my $file = readdir $dh) {
+            if ($file =~ /^\w+\.pm$/) {
+                $file =~ s/\.pm$//;
+                $plugins{$file} = 0;
+                my $module = "Padre::Plugin::$file";
+                #print "loading $module\n";
+                eval "use $module";
+                if ($@) {
+                    warn "ERROR while trying to load plugin '$file': $@";
+                    next;
+                }
+                
+                $plugins{$file} = $module;
+            }
+        }
+    }
+    $self->{plugins} = \%plugins;
+    return;
+}
 
 sub usage {
     die <<"END_USAGE";
@@ -182,7 +231,7 @@ sub run_editor {
 sub _config_dir {
     my $dir = catfile(
 
-              ($ENV{APP_EDITOR_HOME} ? $ENV{APP_EDITOR_HOME} : File::HomeDir->my_data),
+              ($ENV{PADRE_HOME} ? $ENV{PADRE_HOME} : File::HomeDir->my_data),
 
               '.padre');
     if (not -e $dir) {
@@ -258,7 +307,11 @@ sub set_defaults {
     #   last       - the files that were open last time    
     $config->{startup}           ||= 'new';
 
-    $config->{search_term}         = '' if not defined $config->{search_term};
+    #$config->{search_term}         = '' if not defined $config->{search_term};
+    $config->{search_terms}       = [] if not defined $config->{search_terms};
+    if ($config->{search_term}) {
+       $config->{search_terms} = [delete $config->{search_term}]
+    }
 
     $config->{command_line}      ||= '';
     # When running a script from the application some of the files might have not been saved yet.
@@ -479,113 +532,19 @@ sub get_widget {
 
 =head1 BUGS
 
-For now any bug report, please send directly to Gabor Szabo <szabgab@gmail.com>
+Please submit your bugs at L<http://padre.perlide.org/>
 
 =head1 TODO
 
-Send your wish list to Gabor Szabo <szabgab@gmail.com>
+L<http://padre.perlide.org/>
 
 =head2 Editor
 
-  Add unit tests that launch the GUI and generate events.
-
-  Run an application from within the editor:
-  Default: If the files is .pl save automatically and run it with
-     the current perl.
-  Any other file, don't run for now, just display a message.
-  When running a script disable the Run menu and show a stop menu point.
-    (try with a sleeping process)
-  Make sure both STDOUT and STDERR are displayed along with all of their newlines.
-  Allow for STDIN to be redirected to the external process.
-
-  Always save full path in opened-files history.
-  I think the relative pathes were saved when opening files on the
-  command line.
-
   Fix the remaining short-cut key that don't work on Windows (F3)
 
-  Tool for refactoring code: 
-  locate and replace all occurances of a method name within a project.
-
-  When opening a new file if there is an empty unsaved file then replace it.
-  Maybe this should only work on the first such buffer?
-
-  Run an application from within the editor:
-  Simple running works but there are several options here
-
-  - When developing a script we would like to
-    run the current file (the one we currently watch) with some predefined options
-    we should save these options before runs
-  - When developing a larger application with several modules and a script we
-    would like to run a predefined script with some predefind options
-  - Both of the above should be either directly or using a debugger
-  - STDERR and STDOUT should be captured - for simple command line script we might
-    want to have a full command line emulation?
-  - For now we can focus on GUI applications (e.g. written in Wx) where
-    the important part is the STDERR and the debugger.
-  - Run the unit tests assuming Makefile.PL or Build.PL
-
-
-  Project Management:
-
-  We need to assume that the current working directory of the editor can be anywhere.
-  For Perl project management we will assume a directory layout similar to what we have
-  in most CPAN distributions. That is
-
-  lib/    holding the modules
-  bin/    holding the executable scripts
-  t/      holding the test files
-
-  There should be a Makefile.PL or a Build.PL
-  Select a project is basically select a directory.
-  It will try to guess the prefered installation system Build.PL or Makefile.PL
-  Then one should have all the standard steps ready in the menu
-      Project Build.PL, Build, Build test, Build testcover, Build dist Build testdist
-
-  Create a new project will create the directory and the standar files
-  Select a project == select a directory assuming the files are already there
-    or
-  Select a project that was already selected once
-
-    Later we can integrate with various version control system so a new project might start by 
-    pointing to the repository (e.g. svn repo) and checking it out to some local place etc...
-
-  How to keep the path of projects on a stick?
-  Their path can be different in different machines.
-
-
-  Debugger from within the editor
-
-  TAB jump N spaces, clever TAB (real tab in the beginning, spaces after???)
-
-  Mark the buffer that is unsaved with a star.
-
-  Add remote editing capability via ftp and ssh.
-
-  Define API to add more menu options and plugins
-
-  Plugin idea:
-  Based on file type (extension) have a pre-save hook that will include
-  a timestamp in the saved file. E.g. our $TIMESTAMP = DDDDDDDDDD;
-  where DDDDD is the result of time()
-  It could be also a DD.DD.DD number that is automatically increased on every save.
-  With both the main issue is that it is not working if someone 
-  uses a different editor.
-
-  When closing the application and there are still unsaved buffers provide better options:
-  Show the list of the unsaved buffers and the following buttons:
-    Close without saving
-    Cancel  (which will stop the closing)
-    Save  (save those that have filenames, provide save-as option to those that have no filenames)
-    Maybe provide a window with checkboxes for each unsaved buffer so the user can select which one
-    s/he wants to save and which one not.
-
-  Mark a section and comment out all of them at the same time - or remove the # from the beginning
-  of all the lines at once. This works with #, make this work with other comment characters for
-  other languages.
-
-  Keep the size of the output window,
-  Move between the editor and the output window with some hot-key
+  Deal with "resource installation". That is probably talk to
+  Module::Build, Debian and Fedora people to make it easy to install resource files
+  such as xpm or po files. See File::ShareDir.
 
 =head2 Podviewer
 
@@ -656,7 +615,9 @@ Padre::Pod::* are there to index and show documentation written in pod.
 =head1 SUPPORT
 
 I hope the L<http://www.perlmonks.org/> will be ready to take
-upon themself supporting this application. 
+upon themself supporting this application.
+
+See also L<http://padre.perlide.org/>
 
 =head1 COPYRIGHT
 
@@ -677,8 +638,11 @@ that's your problem.
 =head1 CREDITS and THANKS
 
 To Mattia Barbon for providing WxPerl.
-Part of the code was copied from his Wx::Demo application and I think 
-I tool some ideas from Kephra as well.
+Part of the code was copied from his Wx::Demo application.
+
+To Herbert Breunung for leting me work on Kephra.
+
+To Octavian Rasnita for early testing and bug reports.
 
 =cut
 
