@@ -4,12 +4,13 @@ use warnings;
 
 my $output;
 my $run_this_menu;
+my $debug_this_menu;
 my $run_menu;
 my $stop_menu;
 my $proc;
 my $help;
 
-my $status_bar;
+sub say { print @_, "\n" }
 
 my %marker;
 
@@ -22,17 +23,19 @@ use base 'Wx::Frame';
 use Padre::Pod::Frame;
 use Padre::Panel;
 
-use File::Spec::Functions qw(catfile);
+use FindBin;
+use File::Spec::Functions qw(catfile catdir);
 use File::Slurp     qw(read_file write_file);
 use File::Basename  qw(basename fileparse);
 use Carp            qw();
 use Data::Dumper    qw(Dumper);
 use List::Util      qw(max);
-
+use File::ShareDir  ();
 my $default_dir = "";
 our $nb;
 my $cnt = 0;
 
+our $VERSION = '0.01';
 
 # see Wx-0.84/ext/stc/cpp/st_constants.cpp for extension
 my %syntax_of = (
@@ -92,12 +95,25 @@ sub _create_panel {
     $main_panel->SplitHorizontally( $nb, $output, $config->{main}{height} );
 
     $self->CreateStatusBar;
+    my $tool_bar = $self->CreateToolBar( wxTB_HORIZONTAL | wxNO_BORDER | wxTB_FLAT | wxTB_DOCKABLE, 5050);
+    $tool_bar->AddTool(wxID_NEW, '', _bitmap('new'), 'New File');
+    $tool_bar->AddTool(wxID_OPEN, '', _bitmap('open'), 'Open');
+    $tool_bar->AddTool(wxID_SAVE, '', _bitmap('save'), 'Save');
 
     EVT_NOTEBOOK_PAGE_CHANGED($self, $nb, \&on_panel_changed);
 
     return;
 }
 
+sub _bitmap($) {
+    my $file = shift;
+
+    my $dir = $ENV{PADRE_DEV} ? catdir($FindBin::Bin, '..', 'share') : File::ShareDir::dist_dir('Padre');
+    my $path = catfile($dir , 'docview', "$file.xpm" );
+    return Wx::Bitmap->new( $path, wxBITMAP_TYPE_XPM );
+}
+
+
 sub _load_files {
     my ($self) = @_;
 
@@ -130,6 +146,8 @@ sub _load_files {
 sub _create_menu_bar {
     my ($self) = @_;
 
+    my %plugins = %{ $main::app->{plugins} };
+
     my $bar     = Wx::MenuBar->new;
 
     my $file    = Wx::Menu->new;
@@ -137,12 +155,16 @@ sub _create_menu_bar {
     my $view    = Wx::Menu->new;
     my $run     = Wx::Menu->new;
     my $edit    = Wx::Menu->new;
+    my $plugin_menu = Wx::Menu->new;
     my $help    = Wx::Menu->new;
     $bar->Append( $file,    "&File" );
     $bar->Append( $project, "&Project" );
     $bar->Append( $edit,    "&Edit" );
     $bar->Append( $view,    "&View" );
     $bar->Append( $run,     "&Run" );
+    if (%plugins) {
+        $bar->Append( $plugin_menu,     "Pl&ugins" );
+    }
     $bar->Append( $help,    "&Help" );
 
     $self->SetMenuBar( $bar );
@@ -155,7 +177,11 @@ sub _create_menu_bar {
     foreach my $f ($main::app->get_recent('files')) {
        EVT_MENU ($self, $recent->Append(-1, $f), sub { $_[0]->setup_editor($f) } );
     }
-    $file->AppendSubMenu( $recent, "Recent Files" );
+
+    #$file->AppendSubMenu( $recent, "Recent Files" );
+    # to support older version of wxWidgets as well
+    $file->Append( -1, "Recent Files", $recent );
+
     EVT_MENU(  $self, $file->Append( wxID_SAVE,   ''  ), \&on_save    );
     EVT_MENU(  $self, $file->Append( wxID_SAVEAS, ''  ), \&on_save_as );
     EVT_MENU(  $self, $file->Append( -1, 'Save All'  ), \&on_save_all );
@@ -171,8 +197,8 @@ sub _create_menu_bar {
 #    EVT_MENU( $self, $edit->Append( wxID_COPY,    ''       ), \&on_copy    );
 #    EVT_MENU( $self, $edit->Append( wxID_PASTE,   ''       ), \&on_paste   );
     EVT_MENU( $self, $edit->Append( wxID_FIND,    ''       ), \&on_find    );
-    EVT_MENU( $self, $edit->Append( -1,           "&Find Again (F3)"  ), \&on_find_again    );
-    EVT_MENU( $self, $edit->Append( -1,           "&Goto Ctr-G"       ), \&on_goto    );
+    EVT_MENU( $self, $edit->Append( -1,           "&Find Again\tF3"  ), \&on_find_again    );
+    EVT_MENU( $self, $edit->Append( -1,           "&Goto\tCtrl-G"     ), \&on_goto    );
     EVT_MENU( $self, $edit->Append( -1,           "&Setup"      ), \&on_setup   );
 
     my $chk = $view->AppendCheckItem( -1 , "Line numbers" );
@@ -181,18 +207,36 @@ sub _create_menu_bar {
     }
     EVT_MENU( $self, $chk, \&on_toggle_line_numbers);
     EVT_MENU( $self, $view->AppendCheckItem( -1 , "Show Output" ), \&on_toggle_show_output);
+    EVT_MENU( $self, $view->AppendCheckItem( -1 , "Hide StatusBar" ), \&on_toggle_status_bar);
 
-
-    $run_this_menu  = $run->Append( -1 , "Run &This F5" );
-    $run_menu  = $run->Append( -1 , "&Run Any Ctr-F5" );
+    ## Help
+    $run_this_menu  = $run->Append( -1 , "Run &This\tF5" );
+    #$debug_this_menu  = $run->Append( -1 , "Debug This\tF6" );
+    $run_menu  = $run->Append( -1 , "&Run Any\tCtrl-F5" );
     $stop_menu = $run->Append( -1 , "&Stop" );
     EVT_MENU( $self, $run_this_menu,  \&on_run_this);
+    #EVT_MENU( $self, $debug_this_menu,  \&on_debug_this);
     EVT_MENU( $self, $run_menu,  \&on_run);
     EVT_MENU( $self, $stop_menu,  \&on_stop);
     $stop_menu->Enable(0);
 
     EVT_MENU( $self, $run->Append( -1,           "&Setup"      ), \&on_setup_run   );
 
+    ## Plugins
+    foreach my $name (sort keys %plugins) {
+        next if not $plugins{$name};
+        my $submenu = Wx::Menu->new;
+        my @menu = eval {$plugins{$name}->menu;};
+        warn "Error when calling menu for plugin '$name' $@" if $@;
+        foreach my $m (@menu) {
+           EVT_MENU ($self, $submenu->Append(-1, $m->[0]), $m->[1] );
+        }
+        #$plugin_menu->AppendSubMenu( $submenu, $name );
+        $plugin_menu->Append(-1,$name, $submenu);
+    }
+
+
+    ## Help
     EVT_MENU( $self, $help->Append( wxID_ABOUT,   '' ), \&on_about   );
     EVT_MENU( $self, $help->Append( wxID_HELP,    '' ), \&on_help    );
     EVT_MENU( $self, $help->Append( -1,    'Context-help  Ctrl-Shift-H' ), \&on_context_help    );
@@ -213,11 +257,9 @@ sub on_key {
     #print "$mod\n" if $mod;
     #print $event->GetKeyCode, "\n";
     if (not $mod) {
-       if ($code == WXK_F3) {                    # F3
-          $self->on_find_again;
-       } elsif ($code == WXK_F5) {               # F5
-          $self->on_run_this;
-       }
+        if ($code == WXK_F7) {             # F7
+            print "experimental - sending s to debugger\n";
+        }
     } elsif ($mod == 1) {                        # Alt
         if (57 >= $code and $code >= 49) {       # Alt-1-9
             my $id = $code - 49;
@@ -232,13 +274,9 @@ sub on_key {
             $marker{$id} = $line;
 #print "set marker $id to line $line\n";
             #$page->MarkerAdd($line, $id);
-        } elsif ($code == WXK_F5) {               # Ctrl-F5
-            $self->on_run;
         } elsif ($code == WXK_TAB) {              # Ctrl-TAB
             $self->on_next_pane;
-        } elsif ( $code == ord 'G') {             # Ctrl-G
-            $self->on_goto;
-        } elsif ($code == ord 'B') {              # Ctrl-5    Brace matching?
+        } elsif ($code == ord 'B') {              # Ctrl-B    Brace matching?
             my $id   = $nb->GetSelection;
             my $page = $nb->GetPage($id);
             my $pos1  = $page->GetCurrentPos;
@@ -324,7 +362,7 @@ sub on_close_window {
             return;
         }
 
-        my @files = map { _get_filename($_) } (0 .. $nb->GetPageCount -1);
+        my @files = map { $self->_get_filename($_) } (0 .. $nb->GetPageCount -1);
         $config->{main}{files} = \@files;
     }
 
@@ -350,6 +388,8 @@ sub _lexer {
 sub setup_editor {
     my ($self, $file) = @_;
 
+    $self->{_in_setup_editor} = 1;
+
     my $config = $main::app->get_config;
 
 
@@ -364,7 +404,7 @@ sub setup_editor {
 #    print Wx::wxSTC_EOL_LF, "\n";2
 
     $cnt++;
-    my $title = "Unsaved Document $cnt";
+    my $title = " Unsaved Document $cnt";
     my $content = '';
     if ($file) {
         $content = read_file($file);
@@ -380,8 +420,8 @@ sub setup_editor {
     #my $page = $nb->GetCurrentPage;
     my $id  = $nb->GetSelection;
     _set_filename($id, $file);
-    _set_content($id, $content);
 
+    delete $self->{_in_setup_editor};
     $self->update_status;
     return;
 }
@@ -468,34 +508,21 @@ sub _set_filename {
     return;
 }
 sub _get_filename {
-    my ($id) = @_;
+    my ($self, $id) = @_;
 
     my $pack = __PACKAGE__;
     my $page = $nb->GetPage($id);
 
     return $page->{$pack}{filename};
 }
-sub _set_content {
-    my ($id, $data) = @_;
-
-    my $pack = __PACKAGE__;
-    my $page = $nb->GetPage($id);
-    $page->{$pack}{content} = $data;
-
-    return;
-}
-sub _get_content {
-    my ($id) = @_;
-    my $pack = __PACKAGE__;
-    my $page = $nb->GetPage($id);
-    return $page->{$pack}{content};
-}
 
 
 sub on_save_as {
     my ($self) = @_;
 
     my $id   = $nb->GetSelection;
+    return if $id == -1;
+
     while (1) {
         my $dialog = Wx::FileDialog->new( $self, "Save file as...", $default_dir, "", "*.*", wxFD_SAVE);
         if ($dialog->ShowModal == wxID_CANCEL) {
@@ -526,9 +553,11 @@ sub on_save {
     my ($self) = @_;
 
     my $id   = $nb->GetSelection;
-    return if not _buffer_changed($id) and _get_filename($id);
+    return if $id == -1;
 
-    if (_get_filename($id)) {
+    return if not _buffer_changed($id) and $self->_get_filename($id);
+
+    if ($self->_get_filename($id)) {
         $self->_save_buffer($id);
     } else {
         $self->on_save_as();
@@ -551,12 +580,11 @@ sub _save_buffer {
 
     my $page = $nb->GetPage($id);
     my $content = $page->GetText;
-    my $filename = _get_filename($id);
+    my $filename = $self->_get_filename($id);
     eval {
         write_file($filename, $content);
     };
     $main::app->add_to_recent('files', $filename);
-    _set_content($id, $content);
     $nb->SetPageText($id, basename($filename));
     $page->SetSavePoint;
     $self->update_status;
@@ -587,10 +615,7 @@ sub _buffer_changed {
     my ($id) = @_;
 
     my $page = $nb->GetPage($id);
-    #print "$page\n";
-    #print $nb->GetPage($id), "\n";
-    my $content = $page->GetText;
-    return $content ne _get_content($id);
+    return $page->GetModify;
 }
 
 sub on_setup {
@@ -643,7 +668,22 @@ sub on_goto {
     my $id   = $nb->GetSelection;
     my $page = $nb->GetPage($id);
 
-    $page->GotoLine($line_number-1);
+    $line_number--;
+    $page->GotoLine($line_number);
+
+# highlight a line
+#    $page->SetSelection($page->PositionFromLine($line_number), $page->GetLineEndPosition($line_number));
+
+    #$page->SetMarginMask(0, wxSTC_STYLE_LINENUMBER);
+    #$page->SetMarginType(0, wxSTC_STYLE_LINENUMBER);
+
+# put circle next to row
+#    $page->SetMarginWidth(1, 16);
+#    my $fg = Wx::Colour->new( 0xff, 0xff, 0xff );
+#    my $bg = Wx::Colour->new( 0x00, 0x00, 0x00 );
+#    $page->MarkerDefine(0, wxSTC_MARK_CIRCLE, $fg, $bg);
+#    $page->MarkerAdd($line_number, 0);
+#
     return;
 }
 
@@ -651,23 +691,59 @@ sub on_find {
     my ( $self ) = @_;
 
     my $config = $main::app->get_config;
-    my $selection = $self->_get_selection();
 
-    my $search_term = $config->{search_term};
-    if (defined $selection and $selection ne '') { 
-        $search_term = $selection;
-    }
-    my $dialog = Wx::TextEntryDialog->new( $self, "", "Type in search term", $search_term );
+    my $dialog = Wx::Dialog->new( $self, -1, "Search", [-1, -1], [-1, -1]);
+
+    my $box  = Wx::BoxSizer->new(  wxVERTICAL );
+    my $row1 = Wx::BoxSizer->new(  wxHORIZONTAL );
+    my $row2 = Wx::BoxSizer->new(  wxHORIZONTAL );
+    my $row3 = Wx::BoxSizer->new(  wxHORIZONTAL );
+    my $row4 = Wx::BoxSizer->new(  wxHORIZONTAL );
+
+    $box->Add($row1);
+    $box->Add($row2);
+    $box->Add($row3);
+    $box->Add($row4);
+
+    #my @projects = keys %{ $config->{projects} };
+    #push @$search_terms, $search_term if defined $search_term and $search_term ne '';
+
+    my $selection = $self->_get_selection();
+    $selection = '' if not defined $selection;
+    my $choice = Wx::ComboBox->new( $dialog, -1, $selection, [-1, -1], [-1, -1], $config->{search_terms});
+    $row1->Add( $choice, 1, wxALL, 3);
+
+#    $row2->Add($dir_selector, 1, wxALL, 3);
+
+#    my $path = Wx::StaticText->new( $dialog, -1, '');
+#    $row3->Add( $path, 1, wxALL, 3 );
+#    EVT_BUTTON( $dialog, $dir_selector, sub {on_pick_project_dir($path, @_) } );
+    #wxTE_PROCESS_ENTER
+    EVT_TEXT_ENTER($dialog, $choice, sub { $dialog->EndModal(wxID_OK) });
+
+    my $ok     = Wx::Button->new( $dialog, wxID_OK,     '');
+    my $cancel = Wx::Button->new( $dialog, wxID_CANCEL, '');
+    EVT_BUTTON( $dialog, $ok,     sub { $dialog->EndModal(wxID_OK)     } );
+    EVT_BUTTON( $dialog, $cancel, sub { $dialog->EndModal(wxID_CANCEL) } );
+    $row4->Add($cancel, 1, wxALL, 3);
+    $row4->Add($ok,     1, wxALL, 3);
+
+    $dialog->SetSizer($box);
+    #$box->SetSizeHints( $self );
+
+    $choice->SetFocus;
     if ($dialog->ShowModal == wxID_CANCEL) {
         return;
     }
-    $search_term = $dialog->GetValue;
+    my $search_term = $choice->GetValue;
     $dialog->Destroy;
 
-    $search_term = '' if not defined $search_term;
-    return if $search_term eq '';
+    return if not defined $search_term or $search_term eq '';
+
+    unshift @{$config->{search_terms}}, $search_term;
+    my %seen;
+    @{$config->{search_terms}} = grep {!$seen{$_}++} @{$config->{search_terms}};
 
-    $config->{search_term} = $search_term;
     $self->_search();
 
     return;
@@ -677,7 +753,7 @@ sub _search {
     my ($self) = @_;
 
     my $config = $main::app->get_config;
-    my $search_term = $config->{search_term};
+    my $search_term = $config->{search_terms}[0];
 
     my $id   = $nb->GetSelection;
     my $page = $nb->GetPage($id);
@@ -702,7 +778,7 @@ sub on_find_again {
     my ($self) = @_;
 
     my $config = $main::app->get_config;
-    my $search_term = $config->{search_term};
+    my $search_term = $config->{search_terms}[0];
 
     if ($search_term) {
         $self->_search;
@@ -774,7 +850,7 @@ sub on_run_this {
     }
 
     my $id   = $nb->GetSelection;
-    my $filename = _get_filename($id);
+    my $filename = $self->_get_filename($id);
     if (not $filename) {
         Wx::MessageBox( "No filename, cannot run", "Cannot run", wxOK|wxCENTRE, $self );
         return;
@@ -788,15 +864,58 @@ sub on_run_this {
     return;
 }
 
+sub on_debug_this {
+    my ($self) = @_;
+    $self->on_save;
+
+    my $id   = $nb->GetSelection;
+    my $filename = $self->_get_filename($id);
+
+
+    my $host = 'localhost';
+    my $port = 12345;
+
+    _setup_debugger($host, $port);
+
+    local $ENV{PERLDB_OPTS} = "RemotePort=$host:$port";
+    $self->_run("$^X -d $filename");
+
+    return;
+}
+
+# based on remoteport from "Pro Perl Debugging by Richard Foley and Andy Lester"
+sub _setup_debugger {
+    my ($host, $port) = @_;
+
+#use IO::Socket;
+#use Term::ReadLine;
+#
+#    my $term = new Term::ReadLine 'local prompter';
+#
+#    # Open the socket the debugger will connect to.
+#    my $sock = IO::Socket::INET->new(
+#                   LocalHost => $host,
+#                   LocalPort => $port,
+#                   Proto     => 'tcp',
+#                   Listen    => SOMAXCONN,
+#                   Reuse     => 1);
+#    $sock or die "no socket :$!";
+#
+#    my $new_sock = $sock->accept();
+#    my $remote_host = gethostbyaddr($sock->sockaddr(), AF_INET) || 'remote';
+#    my $prompt = "($remote_host)> ";
+}
+
 sub _run {
     my ($self, $cmd) = @_;
+
     $run_this_menu->Enable(0);
     $run_menu->Enable(0);
     $stop_menu->Enable(1);
 
     my $config = $main::app->get_config;
     $main::app->get_widget('main_panel')
-        ->SetSashPosition($config->{main}{height} -100);
+        ->SetSashPosition($config->{main}{height} - 300);
     $output->Remove(0, $output->GetLastPosition);
 
     $proc = Wx::Perl::ProcessStream->OpenProcess($cmd, 'MyName1', $self);
@@ -853,6 +972,17 @@ sub on_toggle_show_output {
     }
 }
 
+sub on_toggle_status_bar {
+    my ($self, $event) = @_;
+
+    my $status_bar = $self->GetStatusBar;
+    if ($event->IsChecked) {
+        $status_bar->Hide;
+    } else {
+        $status_bar->Show;
+    }
+}
+
 sub evt_process_stdout {
     my ($self, $event) = @_;
 
@@ -1089,6 +1219,8 @@ sub on_prev_pane {
 sub update_status {
     my ($self) = @_;
 
+    return if $self->{_in_setup_editor};
+
     my $pageid = $nb->GetSelection();
     if (not defined $pageid) {
         $self->SetStatusText("");
@@ -1096,9 +1228,15 @@ sub update_status {
     }
     my $page = $nb->GetPage($pageid);
     my $line = $page->GetCurrentLine;
-    my $filename = _get_filename($pageid) || '';
+    my $filename = $self->_get_filename($pageid) || '';
     my $modified = $page->GetModify ? '*' : ' ';
 
+    if ($filename) {
+        $nb->SetPageText($pageid, $modified . basename $filename);
+    } else {
+        my $text = substr($nb->GetPageText($pageid), 1);
+        $nb->SetPageText($pageid, $modified . $text);
+    }
     my $pos = $page->GetCurrentPos;
 
     my $start = $page->PositionFromLine($line);
@@ -1114,5 +1252,10 @@ sub on_panel_changed {
     $self->update_status;
 }
 
+sub get_nb {
+    my ($self) = @_;
+    return $nb;
+}
+
 1;
 
